==> approck_sqlalchemy_query_builder-0.1.4.tar.gz: approck_sqlalchemy_query_builder-0.1.4/approck_sqlalchemy_query_builder/__init__.py <==
import datetime
import time
from typing import Dict, List

import pytz
import sqlalchemy as sa
from dateutil.parser import parse
from sqlalchemy.sql.elements import BooleanClauseList, ColumnElement
from sqlalchemy.sql.expression import Select

from approck_sqlalchemy_query_builder.types import Condition, Operator, Query, QueryRule


def build_rule_condition(map_columns: Dict[str, sa.Column], rule: QueryRule) -> ColumnElement[sa.Boolean]:
    column = map_columns.get(rule.id)

    if column is None:
        raise ValueError(f"Column for '{rule.id}' is not found")

    value = rule.value

    if "DATETIME" in str(column.type).upper():
        if isinstance(value, int):
            if value > time.time():
                value /= 1000.0

            value = datetime.datetime.fromtimestamp(value, tz=pytz.utc)
        elif isinstance(value, (str, bytes)):
            value = parse(value).replace(tzinfo=pytz.utc)

    if rule.operator == Operator.OP_EQUAL:
        return column == value
    elif rule.operator == Operator.OP_GREATER_THAN:
        return column > value
    elif rule.operator == Operator.OP_LESS_THAN:
        return column < value
    elif rule.operator == Operator.OP_GREATER_THAN_OR_EQUAL:
        return column >= value
    elif rule.operator == Operator.OP_LESS_THAN_OR_EQUAL:
        return column <= value
    elif rule.operator == Operator.OP_NOT_EQUAL:
        return column != value
    elif rule.operator == Operator.OP_IN:
        return column.in_(value)
    elif rule.operator == Operator.OP_NOT_IN:
        return column.not_in(value)

    raise ValueError(f"Operator '{rule.operator}' is not supported")


def build_condition(
    map_columns: Dict[str, sa.Column], query: Query
) -> BooleanClauseList[ColumnElement[sa.Boolean]] | None:
    clauses: List[BooleanClauseList[ColumnElement[sa.Boolean]] | ColumnElement[sa.Boolean]] = []

    for rule in query.rules:
        if isinstance(rule, Query):
            condition = build_condition(map_columns=map_columns, query=rule)

            if condition is not None:
                clauses.append(condition)
        elif isinstance(rule, QueryRule):
            clauses.append(build_rule_condition(map_columns=map_columns, rule=rule))

    if clauses:
        if query.condition == Condition.OR:
            return sa.or_(*clauses)
        elif query.condition == Condition.AND:
            return sa.and_(*clauses)

    return None


def query_filter(statement: Select, map_columns: Dict[str, sa.Column], query: Query) -> Select:
    condition = build_condition(map_columns=map_columns, query=query)

    if condition is not None:
        statement = statement.where(condition)

    return statement

==> approck_sqlalchemy_query_builder-1.0.1.tar.gz: approck_sqlalchemy_query_builder-1.0.1/approck_sqlalchemy_query_builder/__init__.py <==
import datetime
import time
from typing import Dict, List

import sqlalchemy as sa
from dateutil.parser import parse
from sqlalchemy.sql.elements import ColumnElement
from sqlalchemy.sql.expression import Select

from approck_sqlalchemy_query_builder.types import Condition, Operator, Query, QueryRule


def build_rule_condition(map_columns: Dict[str, sa.Column], rule: QueryRule) -> ColumnElement[sa.Boolean]:
    column = map_columns.get(rule.id)

    if column is None:
        raise ValueError(f"Column for '{rule.id}' is not found")

    value = rule.value

    if "DATETIME" in str(column.type).upper():
        if isinstance(value, int):
            if value > time.time():
                value /= 1000.0

            value = datetime.datetime.fromtimestamp(value, tz=datetime.timezone.utc)
        elif isinstance(value, (str, bytes)):
            value = parse(value).replace(tzinfo=datetime.timezone.utc)

    if rule.operator == Operator.OP_EQUAL:
        return column == value
    elif rule.operator == Operator.OP_GREATER_THAN:
        return column > value
    elif rule.operator == Operator.OP_LESS_THAN:
        return column < value
    elif rule.operator == Operator.OP_GREATER_THAN_OR_EQUAL:
        return column >= value
    elif rule.operator == Operator.OP_LESS_THAN_OR_EQUAL:
        return column <= value
    elif rule.operator == Operator.OP_NOT_EQUAL:
        return column != value
    elif rule.operator == Operator.OP_IN:
        return column.in_(value)
    elif rule.operator == Operator.OP_NOT_IN:
        return column.not_in(value)

    raise ValueError(f"Operator '{rule.operator}' is not supported")


def build_condition(
    map_columns: Dict[str, sa.Column], query: Query
) -> List[ColumnElement[sa.Boolean]] | None:
    clauses: List[ColumnElement[sa.Boolean]] = []

    for rule in query.rules:
        if isinstance(rule, Query):
            condition = build_condition(map_columns=map_columns, query=rule)

            if condition is not None:
                clauses.append(condition)
        elif isinstance(rule, QueryRule):
            clauses.append(build_rule_condition(map_columns=map_columns, rule=rule))

    if clauses:
        if query.condition == Condition.OR:
            return sa.or_(*clauses)
        elif query.condition == Condition.AND:
            return sa.and_(*clauses)

    return None


def query_filter(statement: Select, map_columns: Dict[str, sa.Column], query: Query) -> Select:
    condition = build_condition(map_columns=map_columns, query=query)

    if condition is not None:
        statement = statement.where(condition)

    return statement
